#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import math
import os
from collections import defaultdict
import json

dirpath = "models" # Verzeichnis mit den Parameterdateien

if len(sys.argv) != 2:
    sys.exit('Usage: '+sys.argv[0]+' text-file\n\nThe models are read from the directory "models".')

# Einlesem der verschiedenen Sprachmodelle
print("loading models...", file=sys.stderr)
ngram_prob = {}
backoff = {}
L = 0  # Maximale N-Gramm-Länge (wird beim Einlesen der Modelle bestimmt)
for lang in os.listdir(dirpath):
    with open(os.path.join(dirpath, lang)) as file:
        ngram_prob[lang] = json.load(file)

    # Berechnung der Backoff-Faktoren
    backoff[lang] = defaultdict(lambda: 1.0)
    for ngram, p in ngram_prob[lang].items():
        context = ngram[:-1]
        backoff[lang][context] -= p
        if L < len(ngram):
            L = len(ngram)
print("done", file=sys.stderr)

# Eingabetext einlesen
with open(sys.argv[1]) as file:
    text = file.read()

def smoothed_prob(ngram, lang):
    if len(ngram) == 0:
        # Unigrammwahrscheinlichkeiten werden mit einer uniformen Verteilung geglättet
        return 1.0 / 1000 # Es wird von 1000 möglichen Zeichen ausgegangen
    context = ngram[:-1] # Kontext = N-Gramm ohne letzes Zeichen
    ngram2 = ngram[1:]    # Backoff-NGramm = N-Gramm ohne erstes Zeichen
    p = ngram_prob[lang].get(ngram, 0.0)
    bof = backoff[lang].get(context, 1.0)
    bop = smoothed_prob(ngram2, lang)
    return p + bof * bop

# Am Textanfang Leerzeichen als Kontext für die ersten Buchstaben hinzufügen
text_length = len(text)
text = ' '*(L-1) + text + ' '

# Berechnung der logarithmierten Wahrscheinlichkeit des Textes für jede Sprache
score = {}
for lang in ngram_prob:
    logp = 0.0
    for i in range(text_length+1):
        ngram = text[i:i+L]
        logp += math.log2(smoothed_prob(ngram, lang))
    # berechne Crossentropie = negative logarithmierte Wahrscheinlichkeit 
    # geteilt durch Textlänge
    score[lang] = -logp / text_length

# Sprachen aufsteigend nach Crossentropie sortiert ausgeben
for lang in sorted(score.keys(), key=score.get):
    print(lang, score[lang], sep="\t")
